package com.mowahs.mwcf.framework;

import java.io.Serializable;
import java.util.Vector;
import java.util.Hashtable;
import java.math.BigDecimal;
import java.sql.*;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionError;

/**
 * This class represents an indicator in the MWCF framework. An indicator uses the characterisation
 * of a task to compute values to indicate the complexity in some area of the task and of the
 * scenario as a whole.
 * @author Siv Oma Rogdar
 */
public class IndicatorBean implements Serializable 
{
	//The ID of the indicator in the database
	private int indicatorID;
	//The name of the indicator
	private String name;
	//The description of the indicator
	private String description;
	//Contains the characteristic ids of the main values that the indicator is calculated from.
	private int[] primaryContent;
	//Contains the characterisitc ids of the secondary values that the indicator is calculated from.
	private int[] secondaryContent;
	//All indicators stored in the database. The Vector contains the IndicatorBeans.
	private Vector indicators;
	//The values of this complexity indicator for tasks and scenario as a whole as Floats.
	private Vector indicatorValues;
	
	
	/**
	 */
	public IndicatorBean() 
	{
		
	}
	
	public void setIndicatorID(int id) {
		this.indicatorID = id;
	}
	
	public int getIndicatorID() {
		return this.indicatorID;
	}
	
	/**
	 * Gets the name of the indicator
	 * @return java.lang.String Indicator name
	 */
	public String getName() {
		return this.name;
	}
	
	/**
	 * Sets the name of the indicator
	 * @param name The String name of the indicator
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	/**
	 * Gets the description of the indicator
	 * @return The description
	 */
	public String getDescription() {
		return this.description;
	}
	
	/**
	 * Sets the description of the indicator
	 * @param desc The description
	 */
	public void setDescription(String desc) {
		this.description = desc;
	}
	
	
	/**
	 * Gets the primary content the indicator is calculated from. Returns the vector of primary characteristic ids.
	 * @return int[] The characteristic IDs that are the key characteristics of the indicator
	 */
	public int[] getPrimaryContent() {
		return this.primaryContent;
	}
	
	/**
	 * Sets the primary content the indicator is calculated from. 
	 * @param primCont The array must contain the characteristics' ids.
	 */
	public void setPrimaryContent(int[] primCont) {
		this.primaryContent = primCont;
	}
	
	/**
	 * Gets the secondary content the indicator is calculated from. Returns an array of the characteristic ids.
	 * @return int[]
	 */
	public int[] getSecondaryContent() {
		return this.secondaryContent;
	}
	
	/**
	 * Sets the secondary content the indicator is calculated from. The array must contain the characteristic ids.
	 * @param int[]
	 */
	public void setSecondaryContent(int[] secCont) 
	{
		this.secondaryContent = secCont;
	}
		 
	
	/**
	 * Sets the Vector of all indicators that are stored in the database.
	 * @param java.util.Vector A vector of IndicatorBeans
	 */
	public void setIndicators(Vector indi) {
		this.indicators = indi;
	}
	
	/**
	 * Gets the Vector of all indicators that are stored in the database.
	 * @return java.util.Vector The Vector of IndicatorBeans
	 */
	public Vector getIndicators() {
		return this.indicators;
	}
	
	/**
	 * Sets the Vector of the values of this complexity indicator for tasks and scenario
	 * @param indis A Vector of Floats
	 */
	public void setIndicatorValues(Vector indis) {
		this.indicatorValues = indis;
	}
	
	/**
	 * Gets the Vector of the value of this complexity indicator for tasks and scenario
	 * @return A Vector of Floats
	 */
	public Vector getIndicatorValues() {
		return this.indicatorValues;
	}
	
	/**
	 * Checks if the name of the indicator already exists in the database
	 * 
	 * @param conn A connection with the database
	 * @param errors An ActionErrors to store any errors occurring in
	 * @param name The indicator name to check for
	 * @return False if the name is not unique, true if it is and the indicator can use this name
	 */
	public boolean isNameUnique(Connection conn, ActionErrors errors, String name) {
		boolean unique = false;
		try {	
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery("select INDICATOR_ID from T_INDICATOR where NAME = \""+name+"\"");
			if(rs.next())
				unique = false;
			else 
				unique = true;
				
		} catch(SQLException sqle) {
	       	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("sql.exception"));
   		}	
		return unique;
	}	
	
	/**
	 * Calculates the value of this complexity indicator for a task
	 * @param taskid The id of the task to calculate the indicator for
	 * @param conn A connection with the database
	 * @param errors An ActionErrors to store any errors occurring in
	 * @return The calculated int value of this indicator for the task
	 */
	public float calculateTaskValue(Connection conn, ActionErrors errors, int taskid) 
	{
		double value = 0;
		double tempvalue = 0;
		try {
			Statement stmt = conn.createStatement();
			
			//First get the key characteristics
			ResultSet rs = stmt.executeQuery("select T_TASK_CHAR.VALUE from T_TASK_CHAR, T_INVOLVEMENT where T_TASK_CHAR.CHARACTERISTIC_ID = T_INVOLVEMENT.CHARACTERISTIC_ID and T_INVOLVEMENT.INDICATOR_ID = "+this.indicatorID+" and T_TASK_CHAR.TASK_ID = "+taskid+" and T_INVOLVEMENT.KEY_CHARACTERISTIC = \"y\"");
			int i = 0; 
			while(rs.next()) {
				value = value + (rs.getInt(1)*1000);
				i++;
			}
			if(i>0 && value>0) {
				tempvalue = value;
				value = (value/i);
				
				//If the average is less than 3, get the additional characteristics and include them in the computation
				if(value <= 3000)  {
					
					rs = stmt.executeQuery("select T_TASK_CHAR.VALUE from T_TASK_CHAR, T_INVOLVEMENT where T_TASK_CHAR.CHARACTERISTIC_ID = T_INVOLVEMENT.CHARACTERISTIC_ID and T_INVOLVEMENT.INDICATOR_ID = "+this.indicatorID+" and T_TASK_CHAR.TASK_ID = "+taskid+" and T_INVOLVEMENT.KEY_CHARACTERISTIC = \"n\"");
					while(rs.next()) {
						tempvalue = tempvalue + (rs.getInt(1)*1000);
						i++;
					}
					if(tempvalue>0 && i>0)
						value = (tempvalue/i);					
				}	
			} else {
				value = 0;
			}
		
		} catch(SQLException sqle) {
	       	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("sql.exception"));
   		}	
   		BigDecimal bgtemp = new BigDecimal(value);
   		bgtemp = bgtemp.divide(new BigDecimal(1000), 2, BigDecimal.ROUND_HALF_UP);
		return bgtemp.floatValue();
	}
	 
	/**
	 * Calculates the values of this complexity indicator for a scenario
	 * This method calculates both the indicator values for all tasks in scenario  
	 * and the value for the scenario as a whole. The values are set in the indicatorValues property 
	 * as Float types. The scenario value is the last element of the indicatorValues. This property can 
	 * be retrieved by the getIndicatorValues()-method.
	 * @param scenarioId The id in the database of the scenario to calculate the complexity indicator values from
	 * @param conn A connection with the database
	 * @param errors An ActionErrors to store any occurring errors in
	 */
	public void calculateScenarioValues(Connection conn, ActionErrors errors, Vector tasks) 
	{
		float taskSum = 0;
		int weightSum = 0;
		
		Vector indiValues = new Vector();
		
		for (int i=0; i<tasks.size(); i++) {
			TaskBean task = (TaskBean) tasks.elementAt(i);
			float taskValue = calculateTaskValue(conn, errors, task.getTaskID());
			indiValues.addElement(new Float(taskValue));
			int weight = task.getWeight();
			weightSum = weightSum + weight;
			taskSum = taskSum + taskValue*weight*1000;
		}
		if(taskSum > 0 && weightSum > 0)
			taskSum = taskSum/weightSum;
			
		BigDecimal bgtemp = new BigDecimal(taskSum);
   		bgtemp = bgtemp.divide(new BigDecimal(1000), 2, BigDecimal.ROUND_HALF_UP);
   		indiValues.addElement(new Float(bgtemp.floatValue()));
   		this.setIndicatorValues(indiValues);
	}
	 
	/**
	 * Gets the indicator with the given ID from the database and sets its properties:
	 * name, description, indicatorID, primaryContent and secondaryContent.
	 * 
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 * @param ID The id of the indicator
	 */
	 public void getIndicatorFromDB(Connection conn, ActionErrors errors, int ID) {
	 
		 try {
	   			Statement stmt = conn.createStatement();
	   			ResultSet rs = stmt.executeQuery("select NAME, DESCRIPTION from T_INDICATOR where INDICATOR_ID = "+ID);
						       	
		    	if(rs.next()) {
		    		this.setIndicatorID(ID);
		    		this.setName(rs.getString(1));
					this.setDescription(rs.getString(2));		    		
		    		PreparedStatement stmtCont = conn.prepareStatement("select count(*) from T_INVOLVEMENT where INDICATOR_ID = ? and KEY_CHARACTERISTIC = ?");
					stmtCont.setInt(1, ID);
					stmtCont.setString(2,"y");
					ResultSet res = stmtCont.executeQuery();
					
					PreparedStatement stmtMake = conn.prepareStatement("select CHARACTERISTIC_ID from T_INVOLVEMENT where INDICATOR_ID = ? and KEY_CHARACTERISTIC = ?");
					
					if(res.next()) {
						this.primaryContent = new int[res.getInt(1)];
						stmtMake.setInt(1, ID);
						stmtMake.setString(2,"y");
						ResultSet result = stmtMake.executeQuery();
						
						if(result.next() && (result.getInt(1) != 0)) {
							for(int i = 0; i<this.primaryContent.length; i++) {
								this.primaryContent[i] = result.getInt(1);
								result.next();
							}
						}
							
					}
					
					stmtCont.setInt(1, ID);
					stmtCont.setString(2,"n");
					res = stmtCont.executeQuery();
					
					if(res.next()) {
						this.secondaryContent = new int[res.getInt(1)];
						stmtMake.setInt(1, ID);
						stmtMake.setString(2,"n");
						ResultSet result = stmtMake.executeQuery();
						
						if(result.next() && (result.getInt(1) != 0)) {
							for(int i = 0; i<this.secondaryContent.length; i++) {
								this.secondaryContent[i] = result.getInt(1);
								result.next();
							}
						}		
					}	
				}
        } catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("sql.exception"));
   		}	
   	}
   	
	
	
	/**
	 * Gets all the indicators that are stored in the database and sets a Vector of IndicatorBeans. 
	 * This method gets the names and indicator IDs of the indicators from the database.
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 */ 
	public void getAllIndicatorsFromDB(Connection conn, ActionErrors errors) {
		
		try {
   			Statement stmt = conn.createStatement();
			ResultSet result = stmt.executeQuery("select INDICATOR_ID from T_INDICATOR");
			
			Vector indis = new Vector();
			while(result.next()) {
					IndicatorBean indi = new IndicatorBean();
					indi.getIndicatorFromDB(conn, errors, result.getInt(1));
					indis.addElement(indi);
			}
			this.setIndicators(indis);				
			
        } catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}	
		
	}
	
	/**
	 * Saves the indicator in the database. 
	 * Returns the newly created unique id of the indicator in the database, or -1 if the indicator was not saved.
	 * The IndicatorBean object must have the name, primaryContent and secondaryContent properties set before this method is used.
	 * @return int - The ID of the newly saved indicator
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 */
	public int saveIndicatorInDB(Connection conn, ActionErrors errors) 
	{
		int indicatorID = -1;
		try {
			Statement stmt = conn.createStatement();
			int rows = stmt.executeUpdate("insert into T_INDICATOR (NAME, DESCRIPTION) values (\""+this.name+"\", \""+this.description+"\")");
						
	       	if(rows == 1){
	       		ResultSet rs = stmt.executeQuery("select LAST_INSERT_ID() as lastid");
	       		if(rs.next()) {
	       			indicatorID = rs.getInt("lastid");
	       			this.setIndicatorID(indicatorID);
	       			
	       			PreparedStatement stmtContent = conn.prepareStatement("insert into T_INVOLVEMENT (KEY_CHARACTERISTIC, CHARACTERISTIC_ID, INDICATOR_ID) values (?, ?, ?)");
	       			if(this.primaryContent != null && this.primaryContent.length != 0) {
	       				for(int i = 0; i < this.primaryContent.length; i++) {
	       					stmtContent.setString(1,"y");
	       					stmtContent.setInt(2,this.primaryContent[i]);
	       					stmtContent.setInt(3,indicatorID);
	       					stmtContent.executeUpdate();
	       				}
	       			}
	       			if(this.secondaryContent != null && this.secondaryContent.length != 0) {
	       				for(int i = 0; i < this.secondaryContent.length; i++) {
	       					stmtContent.setString(1,"n");
	       					stmtContent.setInt(2,this.secondaryContent[i]);
	       					stmtContent.setInt(3,indicatorID);
	       					stmtContent.executeUpdate();
	       				}
	       			}
	       			
	       		}
			}
			
        } catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("sql.exception"));
   		}	
		return indicatorID;
	}
	
	/**
	 * Updates the indicators name and description in the database
	 *
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 * @param indicatorId The id of the indicator
	 */
	public void updateIndicatorNameDescriptionInDB(Connection conn, ActionErrors errors, int indicatorId) 
	{
		try {
   			Statement stmt = conn.createStatement();
   			int row = stmt.executeUpdate("update T_INDICATOR set NAME = \""+this.name+"\", DESCRIPTION = \""+this.description+"\" where INDICATOR_ID = "+indicatorId);
			
	    	if(row != 1)
	    		errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("indicator.id.not.exist"));
	    		    	
	    		
	    } catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}
   	}
	
	/**
	 * Updates an indicator's key characteristics and additional characteristics in the database.
	 *
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 * @param indicatorId The id of the indicator
	 */
	public void updateIndicatorInDB(Connection conn, ActionErrors errors, int indicatorId) {
		
		try {
			//First delete all records in T_INVOLVEMENT of this indicator
    		PreparedStatement stmtClear = conn.prepareStatement("delete from T_INVOLVEMENT where INDICATOR_ID = ?");
    		stmtClear.setInt(1, indicatorId);
    		stmtClear.executeUpdate();
    		
    		//Then, insert the new records
			PreparedStatement stmtContent = conn.prepareStatement("insert into T_INVOLVEMENT (KEY_CHARACTERISTIC, CHARACTERISTIC_ID, INDICATOR_ID) values (?, ?, ?)");
   			if(this.primaryContent != null && this.primaryContent.length != 0) {
   				for(int i = 0; i < this.primaryContent.length; i++) {
   					stmtContent.setString(1,"y");
   					stmtContent.setInt(2,this.primaryContent[i]);
   					stmtContent.setInt(3,indicatorId);
   					stmtContent.executeUpdate();
   				}
   			}
   			if(this.secondaryContent != null && this.secondaryContent.length != 0) {
   				for(int i = 0; i < this.secondaryContent.length; i++) {
   					stmtContent.setString(1,"n");
   					stmtContent.setInt(2,this.secondaryContent[i]);
   					stmtContent.setInt(3,indicatorId);
   					stmtContent.executeUpdate();
   				}
   			}		    		
    	
	    } catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}	
		
	}
	
   	/**
   	 * Removes the indicator permanently from the database. 
   	 * Also removes the connection between the characteristics used to calculate the indicator and the indicator.
   	 * @param indicatorID An int specifying the indicator to delete from the database
   	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
   	 */
   	public void removeIndicatorFromDB(Connection conn, ActionErrors errors, int indicatorID) 
	{
		try {
   			Statement stmt = conn.createStatement();
   			stmt.executeUpdate("delete from T_INVOLVEMENT where INDICATOR_ID = "+indicatorID);
						
			int rows = stmt.executeUpdate("delete from T_INDICATOR where INDICATOR_ID = "+indicatorID);
			
			if(rows <= 0)
				errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("database.not.deleted"));
        } catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}	
	}
}
