package com.mowahs.mwcf.framework;

import java.io.Serializable;
import java.util.Vector;
import java.util.Hashtable;
import java.math.BigDecimal;
import org.apache.struts.action.ActionServlet;
import java.sql.*;
import javax.sql.DataSource;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionError;

/**
 * This class represents a mobile scenario in the MWCF framework created by a user. The class
 * handles all logic concerning scenarios, and keeps track of the roles and tasks that belong to the
 * scenario. This class handles all logic tied to scenario creation, storage and deletion.
 * @author Siv Oma Rogdar
 */
public class ScenarioBean implements Serializable 
{
	
	/**
	 * The name of this scenario.
	 */
	private String name = null;
	
	/**
	 * The description of this scenario.
	 */
	private String description = null;
	
	/**
	 * The ID of this scenario, taken from the database. The ID is used as the unique 
	 * identifier of the scenario in the database.
	 */
	private int scenarioID = -1;
	
	/**
	 * The tasks in the scenario.
	 */
	private Vector tasks = null;
	
	/**
	 * The roles of the scenario.
	 */
	
	private Vector roles = null;
	
	/**
	 * The Hashtable of the scenario's role names and their IDs.
	 */
	private Hashtable roleIDs = null;
	
	/**
	 * A Vector of the String names of the tasks of this scenario.
	 */
	private Vector taskNames = null;
	
	/**
	 * A Hashtable of the names (as keys) and the IDs (as values) of the tasks of this scenario.
	 */
	private Hashtable taskNameIDs = null;
	
	//The name or email of the user that created this scenario
	private String creator = null;
	
	//A vector of all scenarios that exist, not directly attached to a logged in user
	//Stored as ScenarioBean objects
	private Vector scenarios = null;
	
	//The average of the weight of those tasks in this scenario that have been assigned a weight
	private float weightAverage = 0;
		
	//The weighted average of all characterisations on all tasks
	private float scenarioAverage;
	
	//The overall mobile scenario complexity indicator
	private float scenarioComplexity;
	
	/**
	 */
	public ScenarioBean() {
		
	}
	
	/**
	 * Get the Vector of TaskBean objects of this scenario.
	 */
	public Vector getTasks() {
	
		return this.tasks;
	
	}
	
	/**
	 * Set the Vector of TaskBean objects of this scenario.
	 */
	public void setTasks(Vector tasks) {
		
		this.tasks = tasks;
		
	}
	
	
	/**
	 * @param name
	 */
	public void setName(String name) 
	{
		this.name = name;
	}
	
	/**
	 * @return java.lang.String
	 */
	public String getName() 
	{
		return this.name;
	}
	
	/**
	 * @param description
	 */
	public void setDescription(String description) 
	{
		this.description = description;
	}
	
	/**
	 * @return java.lang.String
	 */
	public String getDescription() 
	{
		return this.description;
	}
	
	/**
	 * @param scenarioID
	 */
	public void setScenarioID(int scenarioID) 
	{
		this.scenarioID = scenarioID;
	}
	
	/**
	 * @return int scenarioID  The ID of this scenario, as an int.
	 */
	public int getScenarioID() {
	
		return this.scenarioID;	
	}
	
	/**
	 * Gets the Hashtable of rolenames and role ids
	 * @return A Hashtable where the keys are the roles' names and the values are their ids in the database
	 */
	public Hashtable getRoleIDs() {
		return this.roleIDs;
	}

	/**
	 * Sets the Hashtable of role names and role ids
	 * @param roleIds The Hashtable of role names as keys and role ids as values
	 */
	public void setRoleIDs(Hashtable roleIds) {
		this.roleIDs = roleIds;
	}
	
	public String getRoleName(int roleID) {
		
		Integer ID = new Integer(roleID);
		return (String) this.roleIDs.get(ID);
	}
	
	//Sets the name or email of the creator of this scenario
	public void setCreator(String creator) {
		this.creator = creator;
	}
	
	//Gets the name or email of the creator of this scenario
	public String getCreator() {
		return this.creator;
	}
	
	//Sets the Vector of ScenarioBean objects that exist
	public void setScenarios(Vector scenes) {
		this.scenarios = scenes;
	}
	
	//Gets the Vector of ScenarioBean objects
	public Vector getScenarios() {
		return this.scenarios;
	}
	
	/**
	 * Gets the roles of this scenario. Returns a Vector of RoleBean objects.
	 * @return java.util.Vector
	 */
	public Vector getRoles() 
	{
		return this.roles;
	}
	
	public void setRoles(Vector roles) {
		
		this.roles = roles;	
	}
	
	/**
	 * Get the overall scenario complexity indicator
	 * @return The overall scenario complexity indicator value of type float
	 */
	public float getScenarioComplexity() {
		return this.scenarioComplexity;
	}
	
	/**
	 * Set the overall scenario complexity indicator
	 * @param sceneComplex The complexity of the scenario of type float
	 */
	public void setScenarioComplexity(float sceneComplex) {
		this.scenarioComplexity = sceneComplex;
	}
	
	/**
	 * Calculates the overall mobile scenario complexity indicator and sets the scenarioComplexity property
	 * The scenarioAverage property of every CharacteristicBean 
	 * in the MWCF must first have been set. To retrieve the scenario complexity indicator, 
	 * use the getScenarioComplexity()-method.
	 * 
	 * @param characs A Vector of CharacteristicBean objects whose scenarioAverage property have been set
	 */
	 
	public void calculateScenarioComplexity(Vector characs) {
		
		if(!characs.isEmpty()) {
			float characSum = 0;
			
			for(int i=0; i<characs.size(); i++) {
				CharacteristicBean acharac = (CharacteristicBean)characs.elementAt(i);
				float ave = acharac.getScenarioAverage();
				characSum = characSum + ave;			
			}
			if(characSum > 0 && characs.size() > 0)
				characSum = characSum/characs.size();
			
			BigDecimal bg = new BigDecimal(characSum);
			BigDecimal bgtemp = bg.setScale(2, BigDecimal.ROUND_HALF_UP);
   									
			this.setScenarioComplexity(bgtemp.floatValue());
		}
	}
	
	public void setWeightAverage(float ave) {
		this.weightAverage = ave;
	}
	
	public float getWeightAverage() {
		return this.weightAverage;
	}
	
	/** 
	 * Set the value of the average of all characterisation done on all tasks in this scenario
	 * @param ave The float value of the characterisation average
	 */
	public void setScenarioAverage(float ave) {
		this.scenarioAverage = ave;
	}
	
	/**
	 * Get the average of all characterisation done on all tasks of this scenario
	 * @return A float value with 2 decimals, representing the average of all characterisation done on all tasks of this scenario
	 */
	public float getScenarioAverage() {
		return this.scenarioAverage;
	}
	
	/**
	 * Calculates the average of the task weight of this scenario.
	 * May be retrived by the getWeightAverage() method
	 */
	public void calculateWeightAverage() {
		
		Vector temp = this.getTasks();
		float sum = 0;
		
		for(int i=0; i<temp.size(); i++){
		
			TaskBean atask = (TaskBean)temp.elementAt(i);
			if ((atask != null) && (atask.getWeight() > 0))
				sum = sum + atask.getWeight();	
		}
		this.setWeightAverage(sum/temp.size());
	}
				
	/**
	 * Retrieves all the properties of the scenario specified by the scenarioId param, 
	 * from the database. Sets the following properties of the scenario: 
	 * scenarioID, name and description. These properties can be retrieved by getProperty() methods.
	 * 
	 * @param scenarioId An int representing the ID of this scenario in the database.
	 * @return java.lang.String
	 */
	public void getScenarioFromDB(Connection conn, ActionErrors errors, int scenarioId) 
	{
		try {
   			Statement stmt = conn.createStatement();
   			ResultSet rs = stmt.executeQuery("select T_SCENARIO.SCENARIO_ID, T_SCENARIO.NAME, T_SCENARIO.DESCRIPTION, T_USER.NAME, T_USER.EMAIL  from T_SCENARIO, T_USER where SCENARIO_ID = "+scenarioId+" and T_SCENARIO.USER_ID = T_USER.USER_ID");
	       		    
        	if(rs.next()) {
        		this.setScenarioID(rs.getInt(1));
        		this.setName(rs.getString(2));
        		this.setDescription(rs.getString(3));
        		String username = rs.getString(4);
				if(!username.equals("") || username.length() > 0)
					this.setCreator(username);	
				else
					this.setCreator(rs.getString(5));	

        	} else {
        		errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database", name));
    		}
        } catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("database.could.not.connect"));
   		}	
			
	}
	
	/**
	 * Gets all scenarios stored on a specific user from the database.
	 *
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in 
	 * @param userId The ID of the user in the database
	 * @return A Vector of ScenarioBean objects representing the scenarios created by the specified user
	 */
	 
	public Vector getScenariosFromDB(Connection conn, ActionErrors errors, int userId) 
	{
		Vector scenes = new Vector();
		try {
   			Statement stmt = conn.createStatement();
   			ResultSet rs = stmt.executeQuery("select SCENARIO_ID, NAME, DESCRIPTION from T_SCENARIO where USER_ID = "+userId);
            
            while(rs.next()){
            	ScenarioBean scenario = new ScenarioBean();
            	scenario.setScenarioID(rs.getInt(1));
            	scenario.setName(rs.getString(2));
            	scenario.setDescription(rs.getString(3));
            	scenes.addElement(scenario);
            }
        } catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
   		}
   		return scenes;    			
	}
	
	/**
	 * Saves this scenario in the database.
	 *
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 * @param userID The id of the user that created this scenario
	 */
	public int saveScenarioInDB(Connection conn, ActionErrors errors, int userID) 
	{
		this.setScenarioID(-1);
		try {
   			Statement stmt = conn.createStatement();
   			int rows = stmt.executeUpdate("insert into T_SCENARIO (USER_ID, NAME, DESCRIPTION) values ("+userID+", \""+this.name+"\", \""+this.description+"\")");
	       	if(rows == 1) {
	       		ResultSet rs = stmt.executeQuery("select LAST_INSERT_ID() as lastid");
        	   	if(rs.next()) {
        		    this.setScenarioID(rs.getInt("lastid"));
        	          	    
        	    	Hashtable rolesIDs = new Hashtable();
        	    	for(int i=0; i<this.roles.size(); i++) {
        	    		RoleBean role = (RoleBean) roles.elementAt(i);
        	    		int roleID = role.saveRoleGetRoleID(conn, errors, scenarioID);
        	    		Integer id = new Integer(roleID);
        	    	
        	    		if(errors.empty())
        	    			rolesIDs.put(role.getName(), id);
        	    	}
        			this.setRoleIDs(rolesIDs);
        	  
            	} else {
        			errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
    	    	}
		   	} else
		   		errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
		
   		} catch(SQLException sqle) {
	      	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("sql.exception"));
   		}	
   		return this.scenarioID;
	}
	
	/**
	 * Updates the scenario's name and description in the database. 
	 * @param scenarioId The id of this scenario in the database
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 */
	public void updateScenarioInDB(Connection conn, ActionErrors errors, int scenarioId) 
	{
		try {
	       	Statement stmt = conn.createStatement();
	       	int rows = stmt.executeUpdate("update T_SCENARIO set NAME = \""+this.name+"\", DESCRIPTION = \""+this.description+"\" where SCENARIO_ID = "+scenarioId);
	       	
	       	if(rows != 1)
	       		errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database"));
			
   		} catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("sql.exception"));
   		}	
	}	
	
	
	/**
	 * Gets the roles of this scenario from the database. Stores them as RoleBeans. 
	 * Sets the roles variable of this scenario to contain the RoleBeans. The role objects may be retrieved by the getRoles-method.
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 */
	public void getRolesInScenarioFromDB(Connection conn, ActionErrors errors) {
		
		Vector rols = new Vector();
		try {
   			Statement stmt = conn.createStatement();
	       	ResultSet rs = stmt.executeQuery("select ROLE_ID, NAME from T_ROLE where SCENARIO_ID = "+this.scenarioID);
	       	
	       	while(rs.next()) {
	       		RoleBean role = new RoleBean();
	       		role.setRoleID(rs.getInt(1));
	       		role.setName(rs.getString(2));
	       		rols.add(role);	
	       	}
			
   		} catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("database.could.not.connect"));
   		}		
				
		this.roles = rols;
	}
	/**
	 * Adds a RoleBean object to the Vector of roles of this scenario. 
	 * @param role
	 * 
	 */
	public void addRole(RoleBean role) 
	{
		this.roles.addElement(role);	       	
			
	}
	
	/**
	 * Gets all tasks that belong to this scenario from the database.
	 * 
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 */
	public void getTaskNameAndIDsFromDB(Connection conn, ActionErrors errors) {
		
		Hashtable taskNameIDs = new Hashtable();
		Vector taskNames = new Vector();
		try {
   			PreparedStatement stmt = conn.prepareStatement("select ROLE_ID from T_ROLE where SCENARIO_ID = ?");
	       	stmt.setInt(1,this.scenarioID);
	       	ResultSet rs = stmt.executeQuery();
        	
        	while(rs.next()) {
        		int rID = rs.getInt("ROLE_ID");
				PreparedStatement stmtTasks = conn.prepareStatement("select TASK_ID, NAME from T_TASK where ROLE_ID = ?");
				stmtTasks.setInt(1,rID);
				ResultSet rsTasks = stmtTasks.executeQuery();
				
				while(rsTasks.next()) {
					taskNames.add(rsTasks.getString("NAME"));
					taskNameIDs.put(rsTasks.getString("NAME"), new Integer(rsTasks.getInt("TASK_ID")));					
				}
				
			}
			
   		} catch(SQLException sqle) {
	    	errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("database.could.not.connect"));
   		}			
		
		setTaskNames(taskNames);
		setTaskNameIDs(taskNameIDs);
		
	}
	
	
	/**
	 * This method uses the roles of this scenario to create TaskBeans and calls the getTaskWithRoleFromDB-method
	 * on each TaskBean to get their properties. 
	 * Sets the TaskBean objects of this scenario as a Vector of TastBeans. This Vector may be retrieved by the getTasks-method.
	 * 
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 */
	
	public void getTasksInScenarioFromDB(Connection conn, ActionErrors errors) {
		
		Vector tasks = new Vector();
		try	{
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery("select distinct T_TASK.TASK_ID from T_TASK, T_ROLE where T_TASK.ROLE_ID = T_ROLE.ROLE_ID and T_ROLE.SCENARIO_ID = "+this.scenarioID);
			while(rs.next()) {
				TaskBean task = new TaskBean();
				task.getTaskFromDB(conn, errors, rs.getInt(1));
				tasks.add(task);
			}
		} catch(SQLException sqle) {
				errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("sql.exception"));
	   	}	
		this.setTasks(tasks);
	}

	public void setTaskNameIDs(Hashtable taskNameIDs) {
		this.taskNameIDs = taskNameIDs;	
	}
	
	public Hashtable getTaskNameIDs() {
		
		return this.taskNameIDs;
		
	}
	
	public void setTaskNames(Vector taskNames) {
		
		this.taskNames = taskNames;
		
	}
	public Vector getTaskNames() {
		
		return this.taskNames;
		
	}
	
	/**
	 * Removes a scenario permanently from the database. 
	 * This method removes the scenario, all it's roles and tasks, and all characterisation
	 * done from the database.
	 *
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 * @param id The id of the scenario to be deleted.
	 */
	public void removeScenarioFromDB(Connection conn, ActionErrors errors, int id) {
		
		try {
			//Prepared queries that will be used several times
			PreparedStatement gtstmt = conn.prepareStatement("select TASK_ID from T_TASK where ROLE_ID = ?");
			PreparedStatement tcstmt = conn.prepareStatement("delete from T_TASK_CHAR where TASK_ID = ?");
			PreparedStatement tdstmt = conn.prepareStatement("delete from T_TASK_DEPENDENCY where TASK_ID = ?");
			PreparedStatement dtstmt = conn.prepareStatement("delete from T_TASK_DEPENDENCY where DEPENDENT_TASK_ID = ?");
			PreparedStatement depstmt = conn.prepareStatement("delete from T_DEPENDENCIES where ROLE_ID = ?");
			PreparedStatement tstmt = conn.prepareStatement("delete from T_TASK where TASK_ID = ?");
			PreparedStatement rstmt = conn.prepareStatement("delete from T_ROLE where ROLE_ID = ?");
				
			Statement stmt = conn.createStatement();
			//Get the roles of this scenario
			ResultSet rs = stmt.executeQuery("select ROLE_ID from T_ROLE where SCENARIO_ID = "+id);
			
			//For every role in this scenario
			while(rs.next()) {
				int roleID = rs.getInt(1);
				
				//Get the tasks of this role
				gtstmt.setInt(1, roleID);
				ResultSet gtrs = gtstmt.executeQuery();
				
				//For every task of this role
				while(gtrs.next()) {
					int taskID = gtrs.getInt(1);
					
					//Delete all occurrences of this task in the database
					//Delete task from T_TASK_CHAR
					tcstmt.setInt(1, taskID);
					tcstmt.executeUpdate();
					
					//Delete task from T_TASK_DEPENDENCY where task is TASK_ID
					tdstmt.setInt(1, taskID);
					tdstmt.executeUpdate();
					
					//Delete task from T_TASK_DEPENDENCY where task is DEPENDENT_TASK_ID
					dtstmt.setInt(1, taskID);
					dtstmt.executeUpdate();
					
					//Delete task from T_TASK
					tstmt.setInt(1, taskID);
					tstmt.executeUpdate();
				}
				
				//Delete role and task from T_DEPENDENCIES
				depstmt.setInt(1, roleID);
				depstmt.executeUpdate();
				
				//Delete the role from T_ROLE
				rstmt.setInt(1, roleID);
				rstmt.executeUpdate();
			}
			stmt.executeUpdate("delete from T_SCENARIO where SCENARIO_ID = "+id);	
		} catch(SQLException sqle) {
			errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("error.database", sqle));
   		}	
	}
	
	
   /**
	* Gets all scenarios that contain tasks that have been characterised from the database. 
	* Sets the scenarios vector of this object to contain all scenarios found.
	*
	* @param conn A connection with the database
	* @param errors An ActionErrors object to store any occurring errors in
	*/
	public void getAllCharacterisedFromDB(Connection conn, ActionErrors errors) {
	
		try {
			/*	
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery("select distinct T_ROLE.SCENARIO_ID from T_ROLE, T_TASK, T_TASK_CHAR where T_TASK_CHAR.TASK_ID = T_TASK.TASK_ID and T_ROLE.ROLE_ID = T_TASK.ROLE_ID");
			Vector scenaris = new Vector();
			while(rs.next()) {
				ScenarioBean scenario = new ScenarioBean();
				scenario.getScenarioFromDB(conn, errors, rs.getInt(1));
				scenaris.addElement(scenario);
			} 
			this.setScenarios(scenaris);
			*/
			Vector scenaris = new Vector();
			Vector tasks = new Vector();
			int test1, test2, ctest1, ctest2;
			Statement astmt = conn.createStatement();
			ResultSet result = astmt.executeQuery("select distinct scenario_id from T_SCENARIO");
			while(result.next()) {
				test1 = test2 = 0;
				int scenarioid = result.getInt(1);
				Statement stmt = conn.createStatement();
				ResultSet rs = stmt.executeQuery("select distinct T_TASK.task_id from T_TASK, T_ROLE, T_SCENARIO where T_ROLE.role_id = T_TASK.role_id and T_ROLE.scenario_id = "+scenarioid);
				while(rs.next()) {
					test1++;
					}
				//test1 = rs.getFetchSize();
				rs.close();
				stmt.close();
				stmt = conn.createStatement();
				rs = stmt.executeQuery("select distinct T_TASK.task_id from T_TASK, T_ROLE, T_SCENARIO, T_TASK_CHAR where T_ROLE.role_id = T_TASK.role_id and T_TASK.task_id=T_TASK_CHAR.task_id and T_ROLE.scenario_id = "+scenarioid);
				while(rs.next()) {
					ctest1 = ctest2 = 0;
					Statement state = conn.createStatement();
					ResultSet resultchar = state.executeQuery("select characteristic_id from T_CHARACTERISTIC");
					while(resultchar.next())
						ctest1++;
					resultchar.close();
					state.close();
					state = conn.createStatement();
					resultchar = state.executeQuery("select characteristic_id from T_TASK_CHAR where task_id = "+rs.getInt(1));	
					while(resultchar.next())
						ctest2++;
					resultchar.close();
					state.close();
					if(ctest1 == ctest2)
						test2++;
					}
				//test2 = rs.getFetchSize();
				rs.close();
				stmt.close();
				System.out.println("Test values: "+test1+" : "+test2);
				if(test1 == test2) {
				
					Statement stm = conn.createStatement();
					ResultSet res = stm.executeQuery("select SCENARIO_ID, NAME, DESCRIPTION, USER_ID from T_SCENARIO where SCENARIO_ID = "+scenarioid);
					while(res.next()) {
						System.out.println("id4: "+scenarioid);
						ScenarioBean scenario = new ScenarioBean();
						scenario.setScenarioID(res.getInt(1));
						scenario.setName(res.getString(2));
						scenario.setDescription(res.getString(3));
						int user_id = res.getInt(4);
						Statement st = conn.createStatement();
						ResultSet resu = st.executeQuery("select NAME, EMAIL from T_USER where USER_ID = "+user_id);
						if(resu.next()) {
							String na = resu.getString(1);
							if(!na.equals("") || na.length() > 0)
								scenario.setCreator(na);	
							else	
								scenario.setCreator(resu.getString(2));	
							System.out.println("id5: "+scenarioid);
							scenaris.addElement(scenario);
						}
					} 
				}
					
					
				}
				this.setScenarios(scenaris);
		} catch(SQLException sqle) {
			    System.out.println("*****----------********-------------************");
			    System.out.println(sqle.getSQLState());
			    System.out.println("*****----------********-------------************");
				sqle.printStackTrace();
				errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("sql.exception"));
	   	}	
	}
	
	 /**
	* Gets all scenarios of a certain user that contain tasks that have been characterised from the database. 
	* Sets the scenarios vector of this object to contain all scenarios found.
	* @param conn A connection with the database
	* @param errors An ActionErrors object to put any occurring errors in
	* @param userid
	*/
	public void getAllCharacterisedFromDB(Connection conn, ActionErrors errors, int userid) {
		
		Vector scenaris = new Vector();
		System.out.println("id1: "+userid);
		try {	
		/*	Statement astmt = conn.createStatement();
			ResultSet result = astmt.executeQuery("select distinct T_ROLE.ROLE_ID from T_ROLE, T_SCENARIO where T_ROLE.SCENARIO_ID = T_SCENARIO.SCENARIO_ID and T_SCENARIO.USER_ID = "+userid);
			while(result.next()) {
				
				int roleid = result.getInt(1);
				System.out.println("id2: "+userid+" : "+roleid);
				Statement stmt = conn.createStatement();
				ResultSet rs = stmt.executeQuery("select distinct T_TASK.ROLE_ID from T_TASK, T_TASK_CHAR where T_TASK.TASK_ID = T_TASK_CHAR.TASK_ID and T_TASK.ROLE_ID = "+roleid);
				
				while (rs.next()) {
					System.out.println("id3: "+userid+" : "+roleid);
					roleid = rs.getInt(1);
					Statement stm = conn.createStatement();
					ResultSet res = stm.executeQuery("select T_SCENARIO.SCENARIO_ID, T_SCENARIO.NAME, T_SCENARIO.DESCRIPTION, T_SCENARIO.USER_ID from T_SCENARIO, T_ROLE where T_SCENARIO.SCENARIO_ID = T_ROLE.SCENARIO_ID and T_ROLE.ROLE_ID = "+roleid);
					while(res.next()) {
						System.out.println("id4: "+userid+" : "+roleid);
						ScenarioBean scenario = new ScenarioBean();
						scenario.setScenarioID(res.getInt(1));
						scenario.setName(res.getString(2));
						scenario.setDescription(res.getString(3));
						int user_id = res.getInt(4);
						Statement st = conn.createStatement();
						ResultSet resu = st.executeQuery("select NAME, EMAIL from T_USER where USER_ID = "+user_id);
						if(resu.next()) {
							String na = resu.getString(1);
							if(!na.equals("") || na.length() > 0)
								scenario.setCreator(na);	
							else	
								scenario.setCreator(resu.getString(2));	
							System.out.println("id5: "+userid+" : "+roleid);
							scenaris.addElement(scenario);
						} 
						
					}
				}
			}*/
			Vector tasks = new Vector();
			int test1, test2, ctest1, ctest2;
			//Integer temp = new Integer;
			Statement astmt = conn.createStatement();
			ResultSet result = astmt.executeQuery("select distinct T_SCENARIO.scenario_id from T_ROLE, T_SCENARIO where T_SCENARIO.USER_ID = "+userid);
			while(result.next()) {
				test1 = test2 = 0;
				int scenarioid = result.getInt(1);
				Statement stmt = conn.createStatement();
				ResultSet rs = stmt.executeQuery("select distinct T_TASK.task_id from T_TASK, T_ROLE, T_SCENARIO where T_ROLE.role_id = T_TASK.role_id and T_ROLE.scenario_id = "+scenarioid);
				while(rs.next()) {
					test1++;
					}
				//test1 = rs.getFetchSize();
				rs.close();
				stmt.close();
				stmt = conn.createStatement();
				rs = stmt.executeQuery("select distinct T_TASK.task_id from T_TASK, T_ROLE, T_SCENARIO, T_TASK_CHAR where T_ROLE.role_id = T_TASK.role_id and T_TASK.task_id=T_TASK_CHAR.task_id and T_ROLE.scenario_id = "+scenarioid);
				while(rs.next()) {
					ctest1 = ctest2 = 0;
					Statement state = conn.createStatement();
					ResultSet resultchar = state.executeQuery("select characteristic_id from T_CHARACTERISTIC");
					while(resultchar.next())
						ctest1++;
					resultchar.close();
					state.close();
					state = conn.createStatement();
					resultchar = state.executeQuery("select characteristic_id from T_TASK_CHAR where task_id = "+rs.getInt(1));	
					while(resultchar.next())
						ctest2++;
					resultchar.close();
					state.close();
					if(ctest1 == ctest2)
						test2++;
					}
				//test2 = rs.getFetchSize();
				rs.close();
				stmt.close();
				System.out.println("Test values: "+test1+" : "+test2);
				if(test1 == test2) {
				
					Statement stm = conn.createStatement();
					ResultSet res = stm.executeQuery("select SCENARIO_ID, NAME, DESCRIPTION, USER_ID from T_SCENARIO where SCENARIO_ID = "+scenarioid);
					while(res.next()) {
						System.out.println("id4: "+userid+" : "+scenarioid);
						ScenarioBean scenario = new ScenarioBean();
						scenario.setScenarioID(res.getInt(1));
						scenario.setName(res.getString(2));
						scenario.setDescription(res.getString(3));
						int user_id = res.getInt(4);
						Statement st = conn.createStatement();
						ResultSet resu = st.executeQuery("select NAME, EMAIL from T_USER where USER_ID = "+user_id);
						if(resu.next()) {
							String na = resu.getString(1);
							if(!na.equals("") || na.length() > 0)
								scenario.setCreator(na);	
							else	
								scenario.setCreator(resu.getString(2));	
							System.out.println("id5: "+userid+" : ");
							scenaris.addElement(scenario);
						}
					} 
				}
					
					
				}
		} catch(SQLException sqle) {
			errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("sql.exception"));
	   	}
	   	this.setScenarios(scenaris);	
	}
	
	/**
	 * Gets the name, or email if name does not exist, of the user that created this scenario.
	 * Sets the creator property of this scenario.
	 *
	 * @param conn A connection with the database
	 * @param errors An ActionErrors object to store any occurring errors in
	 */
	public void getCreatorFromDB(Connection conn, ActionErrors errors) {
	
		try {	
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery("select T_USER.NAME, T_USER.EMAIL from T_USER, T_SCENARIO where T_SCENARIO.USER_ID = T_USER.USER_ID and T_SCENARIO.SCENARIO_ID = "+this.scenarioID);
			
			if(rs.next()){
				String name = rs.getString(1);
				if(!name.equals("") || name.length() > 0)
					this.setCreator(name);	
				else
					this.setCreator(rs.getString(2));	
			}
		} catch(SQLException sqle) {
				errors.add(ActionErrors.GLOBAL_ERROR, new ActionError("sql.exception"));
	   	}	
	}
	
	
}
